-----------------------------------------------------------
-- LUA Hearts of Iron 3 Production File
-- Modified By: Lothos
-- Date Last Modified: 3/18/2010
-----------------------------------------------------------

function ProductionMinister_Tick(minister)
    --BalanceProductionSliders( minister:GetOwnerAI(), minister:GetCountry() )
    
    if minister:GetOwnerAI():HasFilledProdQueue() then
        ManageProduction(minister)
    end
end

local PRIO_SETTINGS = {
	[0] = { -- full ai automation
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	},
	[1] = {  -- prio production
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	},
	[2] = {   -- prio upgrades
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_UPGRADE_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_PRODUCTION_
	},
	[3] = {   -- prio reinforcement
		[0] = CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	},
	[4] = {   -- prio special reinforcement case
		[0] = CDistributionSetting._PRODUCTION_REINFORCEMENT_,	
		CDistributionSetting._PRODUCTION_CONSUMER_, 
		CDistributionSetting._PRODUCTION_SUPPLY_,
		CDistributionSetting._PRODUCTION_PRODUCTION_,
		CDistributionSetting._PRODUCTION_UPGRADE_
	}
}

function BalanceProductionSliders(ai, ministerCountry, prioSelection)
	--Utils.LUA_DEBUGOUT("BalanceProductionSliders - " .. tostring(ministerCountry:GetCountryTag()) )
		
	-- If country just started mobilizing (or gets bonus reinforcements for some other reason), boost reinforcements
	if ( prioSelection == 0 or prioSelection == 3 )then
		local reinforcement_multiplier = ministerCountry:CalculateReinforcementMultiplier():Get()
		if reinforcement_multiplier > 1.0 then
			prioSelection = 4
		end
	end
	
	local sum = 0
	local changes = { [0] = 0, 0, 0, 0, 0 }	
	
	local totalIC = ministerCountry:GetTotalIC()
	for setting = 0, table.getn( PRIO_SETTINGS[prioSelection] ) do
		local slider = ministerCountry:GetProductionDistributionAt( setting )
		local percentage = slider:GetNeeded():Get() / totalIC
		percentage = percentage 
		changes[ setting ] = math.max(percentage, 0.0 )
	end
	
	-- special considerations
	local dissent = ministerCountry:GetDissent():Get()
	if dissent > 0.01 then -- fight dissent 
		changes[ CDistributionSetting._PRODUCTION_CONSUMER_ ] = changes[ CDistributionSetting._PRODUCTION_CONSUMER_ ] + 0.1
	end
	
	local supplyStockpile = ministerCountry:GetPool():Get( CGoodsPool._SUPPLIES_ ):Get()
	if prioSelection == 0 then
		local weeksSupplyUse = ministerCountry:GetDailyExpense( CGoodsPool._SUPPLIES_ ):Get() * 7
		if (supplyStockpile < weeksSupplyUse * 8.0) then
			changes[ CDistributionSetting._PRODUCTION_SUPPLY_ ] = changes[ CDistributionSetting._PRODUCTION_SUPPLY_ ] + 0.1
		elseif (supplyStockpile > weeksSupplyUse * 20.0) and supplyStockpile > 4000 then
			local newVal = changes[ CDistributionSetting._PRODUCTION_SUPPLY_  ] - 0.1
			if newVal < 0 then
				newVal = 0.0
			end
			changes[ CDistributionSetting._PRODUCTION_SUPPLY_ ] = newVal
		end
	end
	
	if supplyStockpile > 50000 then -- close to cap
		local newVal = changes[ CDistributionSetting._PRODUCTION_SUPPLY_ ] - 0.1
		if newVal < 0 then
			newVal = 0.0
		end
		changes[ CDistributionSetting._PRODUCTION_SUPPLY_ ] = newVal
	end
	
	
	local factor_left = 1
	for i = 0, table.getn( PRIO_SETTINGS[prioSelection] ) do -- normalize
		local index = PRIO_SETTINGS[prioSelection][i]
		local factor = changes[ index ] + 0.00005 -- stop dumb roundoff errors
		factor = math.min(factor, factor_left)
		factor_left = math.max(factor_left - factor, 0.0)

		changes[ index ] = factor
	end
	
	factor_left = math.max(factor_left, 0.0)
	if prioSelection == 0 then
	                
		local neededUpgrades = ministerCountry:GetProductionDistributionAt( CDistributionSetting._PRODUCTION_UPGRADE_ ):GetNeeded():Get()
		local neededPercentage = ((neededUpgrades * 0.1) / totalIC) 
		local currentPercentage =  changes[ CDistributionSetting._PRODUCTION_UPGRADE_ ]
		if currentPercentage < neededPercentage then
			local prodAdd = math.min(factor_left, neededPercentage - currentPercentage)
			factor_left = factor_left - prodAdd
			changes[ CDistributionSetting._PRODUCTION_UPGRADE_ ] = changes[ CDistributionSetting._PRODUCTION_UPGRADE_ ] + prodAdd
		end
	
		changes[ CDistributionSetting._PRODUCTION_PRODUCTION_ ] = changes[ CDistributionSetting._PRODUCTION_PRODUCTION_ ] + factor_left
	else
		if dissent > 0.01
		then
			changes[ CDistributionSetting._PRODUCTION_CONSUMER_ ] = changes[ CDistributionSetting._PRODUCTION_CONSUMER_ ] + factor_left
		else
			changes[ CDistributionSetting._PRODUCTION_SUPPLY_ ] = changes[CDistributionSetting._PRODUCTION_SUPPLY_] + factor_left	
		end
	end
	
	local command = CChangeInvestmentCommand( ministerCountry:GetCountryTag(), changes[0], changes[1], changes[2], changes[3], changes[4] )
	ai:Post( command )
end


-- for debugging
--local GOODS_TO_STRING = { [0] = "_SUPPLIES_","_FUEL_",	"_MONEY_",	"_CRUDE_OIL_",	"_METAL_",	"_ENERGY_",	"_RARE_MATERIALS_" }

-- Create Unit with NO attachments
--    Used to create naval, air and land divisions with no special needs.
function CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, unitTypeString, maxSerial, ic, total_divisions, max_type)
	--Utils.LUA_DEBUGOUT(unitTypeString)
	--Utils.LUA_DEBUGOUT(total_divisions)
	
	local i = 0 -- Counter for amount of units built
	while i < total_divisions do
		local unitType = CSubUnitDataBase.GetSubUnit(unitTypeString)
		local unitcost = ministerCountry:GetBuildCostIC( unitType, 1, bBuildReserveAtPeace ):Get()
		-- Need to add method call here to call brigades processing using the Array
		----  which needs to return a unitType object to be appended
		local buildcount
		
		if 	total_divisions >= (i + maxSerial) then
			buildcount = maxSerial
			i = i + maxSerial
		else
			buildcount = total_divisions - i
			i = total_divisions
		end
		
		local res = ic - (unitcost * max_type)
		if res > 0.2 then
			ic = res
			
			local orderlist = SubUnitList()
			-- Add the amount of brigades requested of the main type
			if max_type == 1 then
				SubUnitList.Append( orderlist, unitType )
			else
				for m = 1, max_type, 1 do
					SubUnitList.Append( orderlist, unitType )
				end
			end

			local construct = CConstructUnitCommand( ministerTag, orderlist, capitalProvId, buildcount, bBuildReserveAtPeace, CNullTag(), CID() )
			ai:Post( construct )
		else
			i = total_divisions --Causes it to exit loop
		end
	end
	
	return ic
end

-- Create Divisions with minor attachments
--    Used only for land untis and adding artillery etc....
function CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, unitTypeString, maxSerial, ic, total_divisions, max_type, MinorUnitArray)
	--Utils.LUA_DEBUGOUT(unitTypeString)
	--Utils.LUA_DEBUGOUT(total_divisions)
	
	local i = 0 -- Counter for amount of units built
	local unitType = CSubUnitDataBase.GetSubUnit(unitTypeString)
	
	if ministerCountry:GetTechnologyStatus():IsUnitAvailable(unitType) then
		local unitcost = ministerCountry:GetBuildCostIC( unitType, 1, bBuildReserveAtPeace ):Get()

		while i < total_divisions do
			local buildcount
			
			if 	total_divisions >= (i + maxSerial) then
				buildcount = maxSerial
				i = i + maxSerial
			else
				buildcount = total_divisions - i
				i = total_divisions
			end
			
			local res = ic - (unitcost * max_type)
			if res > 0.2 then
				ic = res
				
				local orderlist = SubUnitList()
				-- Add the amount of brigades requested of the main type
				if max_type == 1 then
					SubUnitList.Append( orderlist, unitType )
				else
					for m = 1, max_type, 1 do
						SubUnitList.Append( orderlist, unitType )
					end
				end

				-- Attach a minor brigade if one can be attached
				--   updated the ic total for the minor unit being attached
				local TotalMinors = table.getn(MinorUnitArray)
				if TotalMinors == 1 then
					SubUnitList.Append( orderlist, MinorUnitArray[1] )
					ic = ic - (ministerCountry:GetBuildCostIC( MinorUnitArray[1], 1, bBuildReserveAtPeace ):Get())
				elseif TotalMinors > 1 then
					local minorSelected = math.random(TotalMinors)
					SubUnitList.Append( orderlist, MinorUnitArray[minorSelected] )
					ic = ic - (ministerCountry:GetBuildCostIC( MinorUnitArray[minorSelected], 1, bBuildReserveAtPeace ):Get())
				end
				local construct = CConstructUnitCommand( ministerTag, orderlist, capitalProvId, buildcount, bBuildReserveAtPeace, CNullTag(), CID() )
				ai:Post( construct )
			else
				i = total_divisions --Causes it to exit loop
			end
		end
	end

	return ic
end

function ManageProduction(minister)
	local ministerTag = minister:GetCountryTag()
	local ministerTagString = tostring(ministerTag)
	local ministerCountry = minister:GetCountry()
	local ic = ministerCountry:GetICPart( CDistributionSetting._PRODUCTION_PRODUCTION_ ):Get()
	
	ic = ic - ministerCountry:GetUsedIC():Get()

	-- Performance check
	--   if no IC just exit completely so no objects get created
	if ic <= 0.2 then
		return
	end

	local ai = minister:GetOwnerAI() 
	local strategy = ministerCountry:GetStrategy()
	local capitalProvId =  ministerCountry:GetActingCapitalLocation():GetProvinceID()

	-- Contruct convoys and escorts
	ic = ConstructConvoys(ai, minister, ministerTag, ministerCountry, ic )

	-- Performance check (check after convoys)
	--   if no IC just exit completely so no objects get created
	if ic <= 0.2 then
		return
	end
	
	-- Check to make sure they have Manpower
	--    IC check added for performance. If none dont bother executing.
	--Utils.LUA_DEBUGOUT("Country: " .. tostring(ministerTag))
	--Utils.LUA_DEBUGOUT("MP Flag: " .. tostring(ministerCountry:HasExtraManpowerLeft()))
	--Utils.LUA_DEBUGOUT("Actual MP: " .. tonumber(ministerCountry:GetManpower():Get()))

	if ministerCountry:HasExtraManpowerLeft() == true or tonumber(ministerCountry:GetManpower():Get()) > 500 then
		--Utils.LUA_DEBUGOUT("Country: " .. tostring(ministerTag))
		--Utils.LUA_DEBUGOUT("IC: " .. tostring(ministerCountry:GetTotalIC()))
		if ministerCountry:GetTotalIC() <= 50 then
			local bBuildReserveAtPeace = not ministerCountry:IsAtWar()
			local requestQueue = ai:GetReqProdQueue()
			--while (not requestQueue:IsEmpty()) and ic >= 0.0 do
			for request in ai:GetReqProdQueueIter() do
				local unit = request.pUnit --requestQueue:GetTailData().pUnit
				local bBuildReserve = bBuildReserveAtPeace and unit:IsRegiment()
				local tmp = ministerCountry:GetBuildCostIC( unit, 1, bBuildReserve ):Get()
					
				local res = ic / tmp
				if res > 0.2 then
					ic = ic - tmp
					ai:GetReqProdQueue():Remove(request)

					local orderlist = SubUnitList()
					SubUnitList.Append( orderlist, unit )

					local construct = CConstructUnitCommand( ministerTag, orderlist, capitalProvId, 1, bBuildReserve, CNullTag(), CID() )
					ai:Post( construct )
				end
			end
			
			ic = ConstructBuildings(ai, minister, ministerTag, ministerCountry, ic )
		else
			local requestQueue = ai:GetReqProdQueue()
			local bBuildReserveAtPeace = not ministerCountry:IsAtWar()

			-- Setup Potential Minor brigade attachments
			-- ARMORED CARS
			--    Not going to let the AI build armor cars
			-- PARATROOPERS
			--    They get assigned no minor brigade
			local GarrisonUnitArray = {}
			local LegUnitArray = {}
			local MotorizedUnitArray = {}
			local ArmorUnitArray = {}

			-- Leg Brigades
			local anti_air_brigade = CSubUnitDataBase.GetSubUnit("anti_air_brigade")
			local anti_tank_brigade = CSubUnitDataBase.GetSubUnit("anti_tank_brigade")
			local artillery_brigade = CSubUnitDataBase.GetSubUnit("artillery_brigade")
			local engineer_brigade = CSubUnitDataBase.GetSubUnit("engineer_brigade")
			local rocket_artillery_brigade = CSubUnitDataBase.GetSubUnit("rocket_artillery_brigade")

			-- Motorized Brigades
			local sp_artillery_brigade = CSubUnitDataBase.GetSubUnit("sp_artillery_brigade")
			local sp_rct_artillery_brigade = CSubUnitDataBase.GetSubUnit("sp_rct_artillery_brigade")
			local tank_destroyer_brigade = CSubUnitDataBase.GetSubUnit("tank_destroyer_brigade")

			-- Mechanized/Armor Brigades
			local minor_motorized_brigade = CSubUnitDataBase.GetSubUnit("motorized_brigade")
			
			-- Garrison Brigades
			local police_brigade = CSubUnitDataBase.GetSubUnit("police_brigade")


			--Setup potential leg UNIT minor brigades
			-- #############################################
			--Check to see if they can actually build anti_air_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(anti_air_brigade) then
				table.insert( LegUnitArray, anti_air_brigade )
			end
			--Check to see if they can actually build anti_tank_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(anti_tank_brigade) then
				table.insert( LegUnitArray, anti_tank_brigade )
			end
			--Check to see if they can actually build artillery_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(artillery_brigade) then
				table.insert( LegUnitArray, artillery_brigade )
			end
			--Check to see if they can actually build engineer_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(engineer_brigade) then
				table.insert( LegUnitArray, engineer_brigade )
			end		
			--Check to see if they can actually build rocket_artillery_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(rocket_artillery_brigade) then
				table.insert( LegUnitArray, rocket_artillery_brigade )
			end		

			--Setup potential Motorized UNIT minor brigades
			-- #############################################
			--Check to see if they can actually build sp_artillery_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(sp_artillery_brigade) then
				table.insert( MotorizedUnitArray, sp_artillery_brigade )
			end		
			--Check to see if they can actually build sp_rct_artillery_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(sp_rct_artillery_brigade) then
				table.insert( MotorizedUnitArray, sp_rct_artillery_brigade )
			end		
			--Check to see if they can actually build motorized_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(tank_destroyer_brigade) then
				table.insert( MotorizedUnitArray, tank_destroyer_brigade )
			end		

			--Setup potential Armor UNIT minor brigades
			-- #############################################
			--Check to see if they can actually build motorized_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(minor_motorized_brigade) then
				table.insert( ArmorUnitArray, minor_motorized_brigade )
			end		
			
			--Setup potential Garrison UNIT minor brigades
			-- #############################################
			--Check to see if they can actually build police_brigade
			if ministerCountry:GetTechnologyStatus():IsUnitAvailable(police_brigade) then
				table.insert( GarrisonUnitArray, police_brigade )
			end
			
			-- Brigade counts for major units
			local infantry_brigade = 0
			local light_armor_brigade = 0
			local armor_brigade = 0
			local bergsjaeger_brigade = 0
			local cavalry_brigade = 0
			local garrison_brigade = 0
			local heavy_armor_brigade = 0
			local marine_brigade = 0
			local mechanized_brigade = 0
			local militia_brigade = 0
			local motorized_brigade = 0
			local paratrooper_brigade = 0
			local super_heavy_armor_brigade = 0
			
			-- Naval units
			local battlecruiser = 0
			local battleship = 0
			local carrier = 0
			local destroyer = 0
			local escort_carrier = 0
			local heavy_cruiser = 0
			local light_cruiser = 0
			local nuclear_submarine = 0
			local submarine = 0
			local super_heavy_battleship = 0
			local transport = 0
			
			-- Air Units
			local cag = 0
			local cas = 0
			local interceptor = 0
			local multi_role = 0
			local naval_bomber = 0
			local strategic_bomber = 0
			local tactical_bomber = 0
			local transport_plane = 0
			
			-- First loop Determines the counts on how many of each type the AI has determined it needs
			-- If they need enough of a brigade type then build a division.
			for request in ai:GetReqProdQueueIter() do
				-- Start the main code check
				local unit = request.pUnit --requestQueue:GetTailData().pUnit
				local lsUnitType = tostring(unit:GetKey()) -- Get the string type being requested
				
				ai:GetReqProdQueue():Remove(request)
				
				-- Major land unit counts
				if lsUnitType == "infantry_brigade" then
					infantry_brigade = infantry_brigade + 1
				elseif lsUnitType == "light_armor_brigade" then
					light_armor_brigade = light_armor_brigade + 1
				elseif lsUnitType == "armor_brigade" then
					armor_brigade = armor_brigade + 1
				elseif lsUnitType == "bergsjaeger_brigade" then
					bergsjaeger_brigade = bergsjaeger_brigade + 1
				elseif lsUnitType == "cavalry_brigade" then
					cavalry_brigade = cavalry_brigade + 1
				elseif lsUnitType == "garrison_brigade" then
					garrison_brigade = garrison_brigade + 1
				elseif lsUnitType == "heavy_armor_brigade" then
					heavy_armor_brigade = heavy_armor_brigade + 1
				elseif lsUnitType == "marine_brigade" then
					marine_brigade = marine_brigade + 1
				elseif lsUnitType == "mechanized_brigade" then
					mechanized_brigade = mechanized_brigade + 1
				elseif lsUnitType == "militia_brigade" then
					militia_brigade = militia_brigade + 1
				elseif lsUnitType == "motorized_brigade" then
					motorized_brigade = motorized_brigade + 1
				elseif lsUnitType == "paratrooper_brigade" then
					paratrooper_brigade = paratrooper_brigade + 1
				elseif lsUnitType == "super_heavy_armor_brigade" then
					super_heavy_armor_brigade = super_heavy_armor_brigade + 1
				
				-- Start Naval Unit counts
				elseif lsUnitType == "battlecruiser" then
					battlecruiser = battlecruiser + 1
				elseif lsUnitType == "battleship" then
					battleship = battleship + 1
				elseif lsUnitType == "carrier" then
					carrier = carrier + 1
				elseif lsUnitType == "destroyer" then
					destroyer = destroyer + 1
				elseif lsUnitType == "escort_carrier" then
					escort_carrier = escort_carrier + 1
				elseif lsUnitType == "heavy_cruiser" then
					heavy_cruiser = heavy_cruiser + 1
				elseif lsUnitType == "light_cruiser" then
					light_cruiser = light_cruiser + 1
				elseif lsUnitType == "nuclear_submarine" then
					nuclear_submarine = nuclear_submarine + 1
				elseif lsUnitType == "submarine" then
					submarine = submarine + 1
				elseif lsUnitType == "super_heavy_battleship" then
					super_heavy_battleship = super_heavy_battleship + 1
				elseif lsUnitType == "transport_ship" then
					transport = transport + 1
					
				-- Start Air Unit counts
				elseif lsUnitType == "cag" then
					cag = cag + 1
				elseif lsUnitType == "cas" then
					cas = cas + 1
				elseif lsUnitType == "interceptor" then
					interceptor = interceptor + 1
				elseif lsUnitType == "multi_role" then
					multi_role = multi_role + 1
				elseif lsUnitType == "naval_bomber" then
					naval_bomber = naval_bomber + 1
				elseif lsUnitType == "strategic_bomber" then
					strategic_bomber = strategic_bomber + 1
				elseif lsUnitType == "tactical_bomber" then
					tactical_bomber = tactical_bomber + 1
				elseif lsUnitType == "transport_plane" then
					transport_plane = transport_plane + 1

				-- Is there any IC left to even work with
				--    if no just exit the loop (performance)
				elseif ic <= 1 then
					break
					
				-- Unit type is unknown so go ahead and build it
				else 
					Utils.LUA_DEBUGOUT("Unit Type: " .. lsUnitType)
					
					local tmp = ministerCountry:GetBuildCostIC( unit, 1, bBuildReserveAtPeace ):Get()
						
					local res = ic / tmp
					if res > 0.2 then
						ic = ic - tmp

						local orderlist = SubUnitList()
						SubUnitList.Append( orderlist, unit )

						local construct = CConstructUnitCommand( ministerTag, orderlist, capitalProvId, 1, bBuildReserveAtPeace, CNullTag(), CID() )
						ai:Post( construct )
					end
				end
			end

			-- Used in the building of land divisions to figure out how many it needs
			local total_divisions = 0

			
			--Now process naval units counts
			--    always do naval units first as they take the longest to build
			
			-- TRANSPORTS
			--     Serial run limited to 3
			--     first item as no army can move withought them
			if transport > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "transport_ship", 3, ic, transport, 1)
			end		

			--Now process Land unit counts
			--  always check to see if they need 3 or more brigades so full divisions can be formed.
			--  floor command used to round down
			
			-- MILITIA
			if militia_brigade > 2 and ic > 1 then 
				total_divisions = math.floor(militia_brigade / 3)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "militia_brigade", 4, ic, total_divisions, 3, LegUnitArray)
			end		
			
			-- INFANTRY
			if infantry_brigade > 2 and ic > 1 then 
				total_divisions = math.floor(infantry_brigade / 3)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "infantry_brigade", 4, ic, total_divisions, 3, LegUnitArray)
			end
			
			-- GARRISON
			if garrison_brigade > 2 and ic > 1 then 
				total_divisions = math.floor(garrison_brigade / 3)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "garrison_brigade", 4, ic, total_divisions, 3, GarrisonUnitArray)
			end		
			
			-- CAVALRY
			--   CAVALRY units only use 2 unit type attachments instead of 3
			if cavalry_brigade > 1 and ic > 1 then 
				total_divisions = math.floor(cavalry_brigade / 2)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "cavalry_brigade", 6, ic, total_divisions, 2, LegUnitArray)
			end	
			
			-- MOTORIZED INFANTRY
			if motorized_brigade > 2 and ic > 1 then 
				total_divisions = math.floor(motorized_brigade / 3)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "motorized_brigade", 4, ic, total_divisions, 3, MotorizedUnitArray)
			end
			
			-- MECHANIZED
			--   Mechanized units only use 2 unit type attachments instead of 3
			if mechanized_brigade > 1 and ic > 1 then 
				total_divisions = math.floor(mechanized_brigade / 2)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "mechanized_brigade", 3, ic, total_divisions, 2, ArmorUnitArray)
			end

			-- LIGHT ARMOR
			--   Armor units only use 2 unit type attachments instead of 3
			if light_armor_brigade > 1 and ic > 1 then 
				total_divisions = math.floor(light_armor_brigade / 2)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "light_armor_brigade", 2, ic, total_divisions, 2, ArmorUnitArray)
			end
			
			-- MEDIUM ARMOR
			--   Armor units only use 2 unit type attachments instead of 3
			if armor_brigade > 1 and ic > 1 then 
				total_divisions = math.floor(armor_brigade / 2)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "armor_brigade", 2, ic, total_divisions, 2, ArmorUnitArray)			
			end

			-- HEAVY ARMOR
			--   Armor units only use 2 unit type attachments instead of 3
			if heavy_armor_brigade > 1 and ic > 1 then 
				total_divisions = math.floor(heavy_armor_brigade / 2)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "heavy_armor_brigade", 2, ic, total_divisions, 2, ArmorUnitArray)			
			end			
			
			-- SUPER HEAVY ARMOR
			--   Armor units only use 2 unit type attachments instead of 3
			if super_heavy_armor_brigade > 1 and ic > 1 then 
				total_divisions = math.floor(super_heavy_armor_brigade / 2)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "super_heavy_armor_brigade", 2, ic, total_divisions, 2, ArmorUnitArray)			
			end			
			
			-- Now process air unit counts
			--    they are the second longest unit to build.

			-- CAG
			--     Serial run limited to 4
			--		Carries need to get planes first above all else.
			if cag > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "cag", 4, ic, cag, 1)
			end		

			-- INTERCEPTOR
			--     Serial run limited to 4
			if interceptor > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "interceptor", 4, ic, interceptor, 1)
			end		
			
			-- TACTICAL BOMBER
			--     Serial run limited to 4
			if tactical_bomber > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "tactical_bomber", 4, ic, tactical_bomber, 1)
			end		
			
			-- NAVAL BOMBER
			--     Serial run limited to 4
			if naval_bomber > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "naval_bomber", 4, ic, naval_bomber, 1)
			end		

			-- CAS
			--     Serial run limited to 4
			if cas > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "cas", 4, ic, cas, 1)
			end		

			-- MULTI ROLE
			--     Serial run limited to 4
			if multi_role > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "multi_role", 4, ic, multi_role, 1)
			end		

			-- STRATEGIC BOMBER
			--     Serial run limited to 4
			if strategic_bomber > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "strategic_bomber", 4, ic, strategic_bomber, 1)
			end		

			-- TRANSPORT PLANE
			--     Serial run limited to 4
			if transport_plane > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "transport_plane", 4, ic, transport_plane, 1)
			end				

			
			-- Process Naval Units
			-- DESTROYERS
			--     Serial run limited to 3
			if destroyer > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "destroyer", 3, ic, destroyer, 1)
			end		

			-- CRUISER LIGHT
			--     Serial run limited to 2
			if light_cruiser > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "light_cruiser", 2, ic, light_cruiser, 1)
			end		
			
			-- CRUISER HEAVY
			--     Serial run limited to 2
			if heavy_cruiser > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "heavy_cruiser", 2, ic, heavy_cruiser, 1)
			end		
			
			-- CARRIER ESCORT
			--     Serial run limited to 2
			if escort_carrier > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "escort_carrier", 2, ic, escort_carrier, 1)
			end		

			-- CARRIER
			--     Serial run limited to 2
			if carrier > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "carrier", 2, ic, carrier, 1)
			end		

			-- BATTLECRUISER
			--     Serial run limited to 1
			if battlecruiser > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "battlecruiser", 1, ic, battlecruiser, 1)
			end		

			-- BATTLESHIP
			--     Serial run limited to 1
			if battleship > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "battleship", 1, ic, battleship, 1)
			end		

			-- SUPER BATTLESHIP
			--     Serial run limited to 1
			if super_heavy_battleship > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "super_heavy_battleship", 1, ic, super_heavy_battleship, 1)
			end		

			-- SUBMARINE
			--     Serial run limited to 3
			if submarine > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "submarine", 3, ic, submarine, 1)
			end		

			-- NUCLEAR SUBMARINE
			--     Serial run limited to 2
			if nuclear_submarine > 0 and ic > 1 then 
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "nuclear_submarine", 2, ic, nuclear_submarine, 1)
			end		

			
			-- Special forces are processed last once all other needs are met
			
			-- MOUNTAIN
			if bergsjaeger_brigade > 2 and ic > 1 then 
				total_divisions = math.floor(bergsjaeger_brigade / 3)
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "bergsjaeger_brigade", 4, ic, total_divisions, 3)			
			end	
			
			-- PARATROOPER
			if paratrooper_brigade > 2 and ic > 1 then 
				total_divisions = math.floor(paratrooper_brigade / 3)
				ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "paratrooper_brigade", 3, ic, total_divisions, 3)						
			end			
			
			-- MARINES
			if marine_brigade > 2 and ic > 1 then 
				total_divisions = math.floor(marine_brigade / 3)
				ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "marine_brigade", 3, ic, total_divisions, 3, LegUnitArray)	
			end
			
			-- Figure out if the AI asked for units at all
			local TotalUnitsRequested = infantry_brigade + light_armor_brigade + armor_brigade + bergsjaeger_brigade
			TotalUnitsRequested = TotalUnitsRequested + cavalry_brigade + garrison_brigade + heavy_armor_brigade + marine_brigade + mechanized_brigade
			TotalUnitsRequested = TotalUnitsRequested + militia_brigade + motorized_brigade + paratrooper_brigade + super_heavy_armor_brigade
			TotalUnitsRequested = TotalUnitsRequested + battlecruiser + battleship + carrier + destroyer + escort_carrier + heavy_cruiser + light_cruiser
			TotalUnitsRequested = TotalUnitsRequested + nuclear_submarine + submarine + super_heavy_battleship + transport + cag + cas + interceptor
			TotalUnitsRequested = TotalUnitsRequested + multi_role + naval_bomber + strategic_bomber + tactical_bomber + transport_plane
			
			-- Build Extra stuff if left over IC
			--   and the AI is requesting nothing. If it is requesting something dont build anything till the request is met.
			if ic >= 1 and TotalUnitsRequested == 0 then
				
				-- Only build buildings if build requests are met
				ic = ConstructBuildings(ai, minister, ministerTag, ministerCountry, ic )
				
				-- Build units the AI did not request
				--- INFANTRY
				if ic >= 3 then
					ic = CreateDivision(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "infantry_brigade", 4, ic, 4, 3, LegUnitArray)
				end

				--- INTERCEPTORS
				if ic >= 5 then
					ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "interceptor", 4, ic, 2, 1)
				end

				-- TACTICAL BOMBER
				if ic >= 5 then 
					ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "tactical_bomber", 4, ic, 2, 1)
				end		

				-- CAS
				if ic >= 5 then 
					ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "cas", 4, ic, 2, 1)
				end

				--Utils.LUA_DEBUGOUT(tostring(ministerTag))
				--Utils.LUA_DEBUGOUT('Start Naval: ' .. tostring(ic))

				-- Exclude the following countries as they need to concentrate on land stuff
				if ministerTagString ~= "CHI" then
					if ministerCountry:GetNumOfPorts() > 0 then
						-- NAVAL BOMBER
						if ic >= 5 then 
							ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "naval_bomber", 4, ic, 2, 1)
						end		
						
						-- DESTROYERS
						if ic >= 5 then 
							ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "destroyer", 3, ic, 2, 1)
						end		

						-- CRUISER LIGHT
						--     Serial run limited to 2
						if ic >= 5 then 
							ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "light_cruiser", 2, ic, 1, 1)
						end		
					
						-- CRUISER HEAVY
						if ic >= 5 then 
							ic = CreateDivision_nominor(ai, capitalProvId, bBuildReserveAtPeace, ministerTag, ministerCountry, "heavy_cruiser", 2, ic, 1, 1)
						end		
					end
				end
			end			
		end
	else
		-- AI did not have enough manpower to build any units so just do buildings
		ic = ConstructBuildings(ai, minister, ministerTag, ministerCountry, ic )
		
		-- Overide just for the USA, if they are at peace and need some filler for the que since MP is low
		--    build some long term ships as they use little manpower.
		if ministerTagString == "USA" and ic > 1 then
			local ManpowerCount = tonumber(ministerCountry:GetManpower():Get())
		
			-- Make sure USA is not atwar and has over 20 MP and the year is less than 1940
			if not (ministerCountry:IsAtWar()) and ManpowerCount > 100 and tonumber(CCurrentGameState.GetCurrentDate():GetYear()) <= 1939 then
				-- CARRIER
				--     Serial run limited to 2
				if ic > 1 then 
					ic = CreateDivision_nominor(ai, capitalProvId, false, ministerTag, ministerCountry, "carrier", 2, ic, 1, 1)
				end	

				-- BATTLESHIP
				--     Serial run limited to 1
				if ic > 1 then 
					ic = CreateDivision_nominor(ai, capitalProvId, false, ministerTag, ministerCountry, "battleship", 1, ic, 1, 1)
				end	

				-- DESTROYERS
				--     Serial run limited to 3
				if ic > 1 then 
					ic = CreateDivision_nominor(ai, capitalProvId, false, ministerTag, ministerCountry, "destroyer", 3, ic, 3, 1)
				end		

				-- CRUISER LIGHT
				--  Serial run limited to 2
				if ic > 1 then 
					ic = CreateDivision_nominor(ai, capitalProvId, false, ministerTag, ministerCountry, "light_cruiser", 2, ic, 2, 1)
				end		
				
				-- CRUISER HEAVY
				--     Serial run limited to 2
				if ic > 1 then 
					ic = CreateDivision_nominor(ai, capitalProvId, false, ministerTag, ministerCountry, "heavy_cruiser", 2, ic, 2, 1)
				end		
			end
		end
	end
	
	if math.mod( CCurrentGameState.GetAIRand(), 7) == 0 then
		minister:PrioritizeBuildQueue()
	end
end

function ConstructBuildings(ai, minister, ministerTag, ministerCountry, ic)
	-- Buildings
	if ic > 0 then
		-- Get Costal Fort information
		local coastal_fort = CBuildingDataBase.GetBuilding("coastal_fort" )
		local coastal_fortCost = ministerCountry:GetBuildCost(coastal_fort):Get()
		
		-- Get Land Fort Information
		local land_fort = CBuildingDataBase.GetBuilding( "land_fort" )
		local land_fortCost = ministerCountry:GetBuildCost( land_fort ):Get()
		
		local BuildLimitOveride = false

		-- Fist check to see if there are already two being built
		if (ministerCountry:IsMajor()) and tonumber(ministerCountry:GetManpower():Get()) < 500 and tostring(ministerTag) ~= "USA" then
			BuildLimitOveride = true
		end

		
		-- Exclude the following countries as they need to concentrate on land stuff
		if tostring(ministerTag) ~= "CHI" and tostring(ministerTag) ~= "SOV" and tostring(ministerTag) ~= "FRA" then
			--Make sure you wont go into the negative (Performance helper)
			if (ic - coastal_fortCost) >= 0 then
				local totalbuilt = 0
				local TotalBeingBuilt = 0
				
				if BuildLimitOveride == false then
					for navalBaseProvince in ministerCountry:GetNavalBases() do
						if (navalBaseProvince:GetCurrentConstructionLevel(coastal_fort) > 0) then
							TotalBeingBuilt = TotalBeingBuilt + 1
							
							if TotalBeingBuilt >= 2 then
								break
							end
						end
					end
				end
				
				-- If 2 or less then build them
				if TotalBeingBuilt < 2 then	
					-- Just in case it has only one building add it to the count
					totalbuilt = TotalBeingBuilt

					for navalBaseProvince in ministerCountry:GetNavalBases() do
						--Utils.LUA_DEBUGOUT("I am here" )
						if tostring(navalBaseProvince:GetOwner()) == tostring(ministerTag) then
							local provinceBuilding = navalBaseProvince:GetBuilding(coastal_fort)
							
							-- Never put more than 2 coastal forts in a hex
							--    and make sure one is not being built already
							if provinceBuilding:GetMax():Get() < 2 and not (navalBaseProvince:GetCurrentConstructionLevel(coastal_fort) > 0) then
								-- Make sure the hex does not have a fort already (Example: Konigsberg for Germany)
								if not navalBaseProvince:HasBuilding(land_fort) then
									if ministerCountry:IsBuildingAllowed(coastal_fort, navalBaseProvince) then
										local constructCommand = CConstructBuildingCommand( ministerTag, coastal_fort, navalBaseProvince:GetProvinceID(), 1 )

										if constructCommand:IsValid() then
											ai:Post( constructCommand )
											totalbuilt = totalbuilt + 1
											ic = ic - coastal_fortCost -- Upodate IC total
											
											-- Have two building max
											-- or Do not make the second pass your at max ic
											if totalbuilt >= 2 or (ic - coastal_fortCost) <= 0 then
												break 
											end
										end
									end
								end
							end
						end
					end
				end
			end
		end
		
		-- Build Anti Air
		local industry = CBuildingDataBase.GetBuilding("industry" )
		local industryCost = ministerCountry:GetBuildCost(industry):Get()

		local anti_air = CBuildingDataBase.GetBuilding("anti_air" )
		local anti_airCost = ministerCountry:GetBuildCost(anti_air):Get()

		if (ic - anti_airCost) >= 0 then
			local totalbuilt = 0
			local TotalBeingBuilt = 0

			-- Fist check to see if there are already two being built
			if BuildLimitOveride == false then
				for provinceId in ministerCountry:GetOwnedProvinces() do
					local province = CCurrentGameState.GetProvince(provinceId)
					
					if (province:GetCurrentConstructionLevel(anti_air) > 0) then
						TotalBeingBuilt = TotalBeingBuilt + 1
						
						if TotalBeingBuilt >= 2 then
							break
						end
					end
				end
			end
			
			-- If 2 or less then build them
			if TotalBeingBuilt < 2 then
				-- Just in case it has only one building add it to the count
				totalbuilt = TotalBeingBuilt
			
				for provinceId in ministerCountry:GetOwnedProvinces() do
					local province = CCurrentGameState.GetProvince(provinceId)
					local provinceBuilding = province:GetBuilding(anti_air)

					-- First make sure the province has Industry (performance reasons done first)
					if province:HasBuilding(industry) then
						-- Check to see if it has less than 5 AA and nothing is being constructed.
						--    Also make sure its not on the front. If everythign is good then go ahead and build some
						if provinceBuilding:GetMax():Get() < 5 and not (province:GetCurrentConstructionLevel(anti_air) > 0) and not province:IsFrontProvince(false) then
							local constructCommand = CConstructBuildingCommand(ministerTag, anti_air, provinceId, 1 )

							if constructCommand:IsValid() then
								ai:Post( constructCommand )
								totalbuilt = totalbuilt + 1
								ic = ic - anti_airCost -- Upodate IC total	

								-- Have two building max
								--   or Do not make the second pass your at max ic
								if totalbuilt >= 2 or (ic - anti_airCost) <= 0 then
									break 
								end
							end
						end
					end
				end
			end
		end
		
		-- Radar stations
		local radar_station = CBuildingDataBase.GetBuilding("radar_station" )
		local radar_stationCost = ministerCountry:GetBuildCost(radar_station):Get()
		
		if (ic - radar_stationCost) >= 0 then
			local totalbuilt = 0
			local TotalBeingBuilt = 0

			-- Fist check to see if there are already two being built
			if BuildLimitOveride == false then
				for province in ministerCountry:GetAirBases() do
					if (province:GetCurrentConstructionLevel(radar_station) > 0) then
						TotalBeingBuilt = TotalBeingBuilt + 1
						
						if TotalBeingBuilt >= 2 then
							break
						end
					end
				end
			end
			
			-- If 2 or less then build them
			if TotalBeingBuilt < 2 then	
				-- Just in case it has only one building add it to the count
				totalbuilt = TotalBeingBuilt
			
				for province in ministerCountry:GetAirBases() do
					-- Check to make sure the province has factories
					-- Do this check first before any others (performance reasons)
					if province:HasBuilding(industry) and tostring(province:GetOwner()) == tostring(ministerTag) then
						local provinceBuilding = province:GetBuilding(radar_station)
						
						-- Make sure it is not a front province and that the province does not have 2 or more already.
						if provinceBuilding:GetMax():Get() < 2 and not province:IsFrontProvince(false) then
							if ministerCountry:IsBuildingAllowed(radar_station, province) then
								if not (province:GetCurrentConstructionLevel(radar_station) > 0) then
									local constructCommand = CConstructBuildingCommand(ministerTag, radar_station, province:GetProvinceID(), 1)

									if constructCommand:IsValid() then
										ai:Post( constructCommand )
										totalbuilt = totalbuilt + 1
										ic = ic - radar_stationCost -- Upodate IC total	

										-- Have two building max
										--   or Do not make the second pass your at max ic
										if totalbuilt >= 2 or (ic - radar_stationCost) <= 0 then
											break 
										end
									end
								end
							end
						end
					end
				end
			end
		end
	end
	
	return ic
end

function ConstructConvoys(ai, minister, ministerTag, ministerCountry, ic)
	local TransportsNeeded = ministerCountry:GetTotalNeededTransports()
	local TransportsCurrent = ministerCountry:GetTransports()
	local PortCount = ministerCountry:GetNumOfPorts()
	
	if TransportsNeeded > TransportsCurrent and PortCount > 0 then
		local TransportConstruction = minister:CountTransportsUnderConstruction()
		local TransportsActuallyNeeded = TransportsNeeded - TransportsCurrent - TransportConstruction
		local maxSerial
		
		-- Majors build 20% more than you need
		if (ministerCountry:IsMajor()) then
			maxSerial = 4
			TransportsActuallyNeeded = ((TransportsNeeded - TransportsCurrent - TransportConstruction) * 1.20)

		-- Minors just build exactly what you need or close to it
		--   - they also do shorter runs since they needed resources more than majors
		else
			maxSerial = 2
			TransportsActuallyNeeded = TransportsNeeded - TransportsCurrent - TransportConstruction
		end
		
		if TransportsActuallyNeeded > 0 then
			local cost = ministerCountry:GetConvoyBuildCost():Get()
			local buildRequestCount = TransportsActuallyNeeded / defines.economy.CONVOY_CONSTRUCTION_SIZE
			buildRequestCount = math.ceil( math.max( buildRequestCount, 1) )
			ic = BuildTransportOrEscort(ai, ministerTag, buildRequestCount, maxSerial, false, cost, ic)
		end
	end
	
	-- Now Process Escorts Check
	--   Performance check make sure they have IC to actually work with
	if ic > 0 then
		local EscortsNeeded = minister:CountTotalDesiredEscorts()
		local EscortsCurrent = ministerCountry:GetEscorts()

		if EscortsNeeded > EscortsCurrent and PortCount > 0 then
			local EscortsConstruction = minister:CountEscortsUnderConstruction()
			local EscortsActuallyNeeded = EscortsNeeded - EscortsCurrent - EscortsConstruction

			if EscortsActuallyNeeded > 0 then
				local cost = ministerCountry:GetEscortBuildCost():Get()
				local buildRequestCount = EscortsActuallyNeeded / defines.economy.CONVOY_CONSTRUCTION_SIZE
				buildRequestCount = math.ceil( math.max( buildRequestCount, 1) )
				ic = BuildTransportOrEscort(ai, ministerTag, buildRequestCount, 4, true, cost, ic)
			end 
		end
	end
	
	return ic
end

--ConvoyOrEscort = is a boolean (true = escort, false = convoy)
function BuildTransportOrEscort(ai, ministerTag, total_transports, maxSerial, ConvoyOrEscort, cost, ic)
	local i = 0 -- Counter for amount of units built
	
	while i < total_transports do
		-- Need to add method call here to call brigades processing using the Array
		----  which needs to return a unitType object to be appended
		local buildcount
		
		if 	total_transports >= (i + maxSerial) then
			buildcount = maxSerial
			i = i + maxSerial
		else
			buildcount = total_transports - i
			i = total_transports
		end
		
		local res = ic - cost

		if res > 0 then
			local buildCommand = CConstructConvoyCommand(ministerTag, ConvoyOrEscort, buildcount)
			ai:Post(buildCommand)
			
			ic = ic - cost
		end
	end
	
	return ic
end