-----------------------------------------------------------
-- LUA Hearts of Iron 3 Research File
-- Modified By: Lothos
-- Date Last Modified: 3/18/2010
-----------------------------------------------------------

function BalanceLeadershipSliders(ai, ministerCountry)
	-- Setup default sliders
	local currentDiplomats = ministerCountry:GetDiplomaticInfluence():Get()
	local isMajor =  ministerCountry:IsMajor()

	local LEADERSHIP_RESEARCH
	local LEADERSHIP_ESPIONAGE = 0.08
	local LEADERSHIP_DIPLOMACY = 0.15
	local LEADERSHIP_NCO = 0.1
	
	-- Officer ratio.
	local officer_ratio = ministerCountry:GetOfficerRatio():Get()
		
	-- Checks to see if you are loosing officers
	--   if so take them from research
	if  officer_ratio < 0.7 then
		LEADERSHIP_NCO = 0.5
	elseif  officer_ratio < 0.9 then
		LEADERSHIP_NCO = 0.3
	elseif officer_ratio  < 1.2 then
		LEADERSHIP_NCO = 0.2
	
	-- Check to see if you have to many officers
	--    if so increase research
	elseif officer_ratio > 1.4 then
		LEADERSHIP_NCO = 0.0
	end
	
	-- If the AI has to many diplomats then set it to 0
	if isMajor == true and currentDiplomats >= 100 or isMajor == false and currentDiplomats >= 25 then
		LEADERSHIP_DIPLOMACY = 0
	-- AI is loosing to many diplomats double what the default is
	elseif currentDiplomats == 0 then
		LEADERSHIP_DIPLOMACY = LEADERSHIP_DIPLOMACY + LEADERSHIP_DIPLOMACY
	end

	-- Research is whatever is left over
	LEADERSHIP_RESEARCH = (((1 - LEADERSHIP_ESPIONAGE) - LEADERSHIP_DIPLOMACY) - LEADERSHIP_NCO)
	
	local command = CChangeLeadershipCommand( ministerCountry:GetCountryTag(), LEADERSHIP_NCO, LEADERSHIP_DIPLOMACY, LEADERSHIP_ESPIONAGE, LEADERSHIP_RESEARCH)
	ai:Post( command )
	
	return LEADERSHIP_RESEARCH
end

function TechMinister_Tick(minister)
	local ai = minister:GetOwnerAI()
	local ministerCountry = minister:GetCountry()

	-- Calling balance sliders like this allows me to get what the new Research slot count would be
	--    once the sliders are shifted
	local ResearchSlotsAllowed = ministerCountry:GetTotalLeadership() * BalanceLeadershipSliders(ai, ministerCountry)

	-- This command is needed incase it gets convereted by LUA
	--  into userdata do to being a float!
	local ResearchSlotsNeeded = tonumber(tostring(ResearchSlotsAllowed - ministerCountry:GetNumberOfCurrentResearch()))
	
	Process_Tech((CCurrentGameState.GetCurrentDate():GetYear()), ResearchSlotsAllowed, ResearchSlotsNeeded, ai, minister, ministerCountry)
end

-- Processes the main tech reasearch for the specified country
--   designed to be a recursive call in case the AI needs to research in the future
function Process_Tech(pYear, ResearchSlotsAllowed, ResearchSlotsNeeded, ai, minister, ministerCountry)
	-- Performance check, exit if there are no slots available
	if ResearchSlotsNeeded <= 0 then
		return
	end
	
	local ministerTag = minister:GetCountryTag()
	local ministerTagString = tostring(ministerTag)
	local techStatus = ministerCountry:GetTechnologyStatus()
	local isMajor =  ministerCountry:IsMajor()
	local TotalIC = ministerCountry:GetTotalIC()
		
	--Utils.LUA_DEBUGOUT("Country: " .. tostring(ministerTag))
		
	-- List of Arrays to hold the techs the AI can research
	local landBased = {}
	local landDoctrines = {}
	local airBased = {}
	local airDoctrines = {}
	local navalBased = {}
	local navalDoctrines = {}
	local industrial = {}
	local secretWeapons = {}
	local other = {}
	
	-- Counts to hold what the AI is currently researching
	local landBasedCount = 0
	local landDoctrinesCount = 0
	local airBasedCount = 0
	local airDoctrinesCount = 0
	local navalBasedCount = 0
	local navalDoctrinesCount = 0
	local industrialCount = 0
	local secretWeaponsCount = 0
	local otherCount = 0
	
	
	-- Weight system % based 1.0 = 100%
	local landBasedWeight = 0
	local landDoctrinesWeight = 0
	local airBasedWeight = 0
	local airDoctrinesWeight = 0
	local navalBasedWeight = 0
	local navalDoctrinesWeight = 0
	local industrialWeight = 0
	local secretWeaponsWeight = 0
	local otherWeight = 0
	
	-- Flag indicating if they should research only artilery types
	local artilleryOnly = true

	-- First determine research weights
	if isMajor then
		-- Make sure all major powers will research armor
		artilleryOnly = false
		
		-- Land based Major powers Germany, France and USSR
		if ministerTagString == "GER" or ministerTagString == "FRA" or ministerTagString == "SOV" then
			landBasedWeight = 0.24
			landDoctrinesWeight = 0.18
			airBasedWeight = 0.11
			airDoctrinesWeight = 0.18
			navalBasedWeight = 0.06
			navalDoctrinesWeight = 0.05
			industrialWeight = 0.10
			secretWeaponsWeight = 0.04
			otherWeight = 0.04
		
		-- Combination of Naval and Land Major Powers Japan, USA, UK and Italy
		elseif ministerTagString == "JAP" or ministerTagString == "USA" or ministerTagString == "ENG" or ministerTagString == "ITA" then
			landBasedWeight = 0.18
			landDoctrinesWeight = 0.09
			airBasedWeight = 0.12
			airDoctrinesWeight = 0.15
			navalBasedWeight = 0.18
			navalDoctrinesWeight = 0.10
			industrialWeight = 0.10
			secretWeaponsWeight = 0.04
			otherWeight = 0.04
			
			-- Japan will only research artillery techs and not armor techs
			if ministerTagString == "JAP" then
				artilleryOnly = true
			end
			
		-- Just in case new Major Powers get declared
		else
			landBasedWeight = 0.26
			landDoctrinesWeight = 0.10
			airBasedWeight = 0.20
			airDoctrinesWeight = 0.15
			navalBasedWeight = 0.06
			navalDoctrinesWeight = 0.05
			industrialWeight = 0.10
			secretWeaponsWeight = 0.04
			otherWeight = 0.04		
		end
	
	-- Check to see if the minor has any ports and if it has more than 20 IC
	elseif ministerCountry:GetNumOfPorts() > 0 and TotalIC >= 20 then
		landBasedWeight = 0.20
		landDoctrinesWeight = 0.09
		airBasedWeight = 0.12
		airDoctrinesWeight = 0.15
		navalBasedWeight = 0.18
		navalDoctrinesWeight = 0.10
		industrialWeight = 0.14
		secretWeaponsWeight = 0.00
		otherWeight = 0.02
	
		-- If they have more than 50 IC let them research armor
		if TotalIC >= 50 then
			artilleryOnly = false
		end
		
	-- If the minor has no ports and less than 20 IC then concentrate on land techs
	else
		landBasedWeight = 0.30
		landDoctrinesWeight = 0.15
		airBasedWeight = 0.20
		airDoctrinesWeight = 0.15
		navalBasedWeight = 0.0
		navalDoctrinesWeight = 0.0
		industrialWeight = 0.15
		secretWeaponsWeight = 0.00
		otherWeight = 0.05		

		-- If they have more than 50 IC let them research armor
		if TotalIC >= 50 then
			artilleryOnly = false
		end
	end	
	
	-- Figure out what the AI currently is researching
	local techFolder
	
	for tech in ministerCountry:GetCurrentResearch() do
		techFolder = tostring(tech:GetFolder():GetKey())
		
		if techFolder == "infantry_folder" or techFolder == "armour_folder" then
			landBasedCount = landBasedCount + 1
		elseif techFolder == "land_doctrine_folder" then
			landDoctrinesCount = landDoctrinesCount + 1
		elseif techFolder == "fighter_folder" or techFolder == "bomber_folder" then
			airBasedCount = airBasedCount + 1
		elseif techFolder == "air_doctrine_folder" then
			airDoctrinesCount = airDoctrinesCount + 1
		elseif techFolder == "smallship_folder" or techFolder == "capitalship_folder" then
			navalBasedCount = navalBasedCount + 1
		elseif techFolder == "naval_doctrine_folder" then
			tnavalDoctrinesCount = navalDoctrinesCount + 1
		elseif techFolder == "industry_folder" then
			industrialCount = industrialCount + 1
		elseif techFolder == "secretweapon_folder" then
			secretWeaponsCount = secretWeaponsCount + 1
		else
			otherCount = otherCount + 1
		end		
	end
	
	-- Figure out what the AI can research
	for tech in CTechnologyDataBase.GetTechnologies() do
		if  minister:CanResearch(tech) and tech:IsValid() then
			local nYear = techStatus:GetYear(tech, (techStatus:GetLevel(tech) + 1))
			
			-- Concentrate only on techs for the year requested or less
			--- Penalties are way to high to go into the future
			if nYear <= pYear then
				techFolder = tostring(tech:GetFolder():GetKey())
				
				if techFolder == "infantry_folder" then
					local techName = tostring(tech:GetKey())
					
					-- Ignore cavalry techs as the only thing they do is help with practicle for motorized
					if techName ~= "cavalry_smallarms" and techName ~= "cavalry_support" and techName ~= "cavalry_guns" and techName ~= "cavalry_at" then
						table.insert( landBased, tech )
					end
					
					
					--if Utils.HasCountryAIFunction( ministerTag, 'IgnoreLandTech' ) then
					--	return Utils.CallCountryAI( ministerTag, 'IgnoreLandTech', ai, minister, countryTag, bestMission, bestScore )
					--else
					--	call the default one
					--end
					
				elseif techFolder == "armour_folder" then
					local techName = tostring(tech:GetKey())
					
					-- Ignore Armor cars all together
					if techName ~= "armored_car_armour" and techName ~= "armored_car_gun" then
						if artilleryOnly == true then
							-- Only research the following artillery type techs
							if techName == "art_barrell_ammo" or techName == "art_carriage_sights" or techName == "at_barrell_sights" or techName == "at_ammo_muzzel" or techName == "aa_barrell_ammo" or techName == "aa_carriage_sights" or techName == "rocket_art" or techName == "rocket_art_ammo" or techName == "rocket_carriage_sights" then
								table.insert( landBased, tech )
							end
						else
							table.insert( landBased, tech )
						end
					end
				elseif techFolder == "land_doctrine_folder" then
					table.insert( landDoctrines, tech )
				elseif techFolder == "fighter_folder" or techFolder == "bomber_folder" then
					table.insert( airBased, tech )
				elseif techFolder == "air_doctrine_folder" then
					table.insert( airDoctrines, tech )
				elseif techFolder == "smallship_folder" or techFolder == "capitalship_folder" then
					table.insert( navalBased, tech )
				elseif techFolder == "naval_doctrine_folder" then
					table.insert( navalDoctrines, tech )
				elseif techFolder == "industry_folder" then
					table.insert( industrial, tech )
				elseif techFolder == "secretweapon_folder" then
					table.insert( secretWeapons, tech )
				elseif techFolder == "theory_folder" then
					local techName = tostring(tech:GetKey())
					
					-- Do not let the AI  waste its time on theory except for the supply stuff
					if techName == "supply_transportation" or techName == "supply_organisation" or techName == "civil_defence" then
						table.insert( other, tech )
					end
				else
					table.insert( other, tech )
				end
			end
		end
	end
	
	-- Calculate what the AI wants to research in each category based on the weights
	---  AI may put more slots in that it can research but thats no big deal
	local _landBased = CalculateTechSlots(((ResearchSlotsAllowed * landBasedWeight) - landBasedCount))
	local _landDoctrines = CalculateTechSlots(((ResearchSlotsAllowed * landDoctrinesWeight) - landDoctrinesCount))
	local _airBased = CalculateTechSlots(((ResearchSlotsAllowed * airBasedWeight) - airBasedCount))
	local _airDoctrines = CalculateTechSlots(((ResearchSlotsAllowed * airDoctrinesWeight) - airDoctrinesCount))
	local _navalBased = CalculateTechSlots(((ResearchSlotsAllowed * navalBasedWeight) - navalBasedCount))
	local _navalDoctrines = CalculateTechSlots(((ResearchSlotsAllowed * navalDoctrinesWeight) - navalDoctrinesCount))
	local _industrial = CalculateTechSlots(((ResearchSlotsAllowed * industrialWeight) - industrialCount))
	local _secretWeapons = CalculateTechSlots(((ResearchSlotsAllowed * secretWeaponsWeight) - secretWeaponsCount))
	local _other = CalculateTechSlots(((ResearchSlotsAllowed * otherWeight) - otherCount))
	
	-- Holds extra research slots that the AI is unable to use
	local ExtraSlots = ResearchSlotsNeeded - _landBased - _landDoctrines - _airBased - _airDoctrines - _navalBased - _navalDoctrines - _industrial - _secretWeapons - _other
	
	-- First figure out if there are any Extra slots going to be left
	ExtraSlots = GetExtraSlots(ExtraSlots, _landBased, landBased)
	ExtraSlots = GetExtraSlots(ExtraSlots, _landDoctrines, landDoctrines)
	ExtraSlots = GetExtraSlots(ExtraSlots, _airBased, airBased)
	ExtraSlots = GetExtraSlots(ExtraSlots, _airDoctrines, airDoctrines)
	ExtraSlots = GetExtraSlots(ExtraSlots, _navalBased, navalBased)
	ExtraSlots = GetExtraSlots(ExtraSlots, _navalDoctrines, navalDoctrines)
	ExtraSlots = GetExtraSlots(ExtraSlots, _industrial, industrial)
	ExtraSlots = GetExtraSlots(ExtraSlots, _secretWeapons, secretWeapons)
	ExtraSlots = GetExtraSlots(ExtraSlots, _other, other)

	-- Now Reseach each
	landBased = ResearchTech(ai, ministerTag, _landBased, landBased)
	landDoctrines = ResearchTech(ai, ministerTag, _landDoctrines, landDoctrines)
	airBased = ResearchTech(ai, ministerTag, _airBased, airBased)
	airDoctrines = ResearchTech(ai, ministerTag, _airDoctrines, airDoctrines)
	navalBased = ResearchTech(ai, ministerTag, _navalBased, navalBased)
	navalDoctrines = ResearchTech(ai, ministerTag, _navalDoctrines, navalDoctrines)
	industrial = ResearchTech(ai, ministerTag, _industrial, industrial)
	secretWeapons = ResearchTech(ai, ministerTag, _secretWeapons, secretWeapons)
	other = ResearchTech(ai, ministerTag, _other, other)
	
	-- Now process the extra slots
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, industrial)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, landBased)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, airBased)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, navalBased)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, landDoctrines)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, airDoctrines)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, navalDoctrines)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, secretWeapons)
	end
	if ExtraSlots > 0 then
		ExtraSlots = ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, other)
	end
	
	if ExtraSlots > 0 then
		-- We have extra slots and no techs to research so go ahead and look into the future.
		Process_Tech((pYear + 1), ResearchSlotsAllowed, ExtraSlots, ai, minister, ministerCountry)
	end
end

-- Rounding method < 0.5 round down >= to 0.5 round up
function CalculateTechSlots(ResearchCount)
	local rValue
	
	-- This command is needed incase it gets convereted by LUA
	--  into userdata do to being a float!
	ResearchCount = tonumber(tostring(ResearchCount))
	
	if (ResearchCount - math.floor(ResearchCount)) >= 0.5 then
		rValue = math.ceil(ResearchCount)
	else
		rValue = math.floor(ResearchCount)
	end
	
	if rValue < 0 then
		return 0
	else
		return rValue
	end
end

-- Select a random tech from the array
function ResearchTech(ai, ministerTag, ResearchCount, TechArray)
	-- Performance check 
	--    its possible this method gets called but the AI doesnt want to research anything in this area
	if ResearchCount == 0 then
		return TechArray
	end

	local TableLength = table.getn(TechArray)
	local TechSelected
	
	-- Just makes sure it can loop with no issues, the GetExtraSlots already account for it being larger
	if TableLength == 0 then
		return TechArray
	elseif ResearchCount > TableLength then
		ResearchCount = TableLength
	end
	
	local i = 0
	while i < ResearchCount do
		TechSelected = math.random(TableLength)

		local command = CStartResearchCommand( ministerTag, TechArray[TechSelected] )
		ai:Post( command )
		
		-- Remove the tech from the array
		table.remove(TechArray, TechSelected)
		-- Update the TabelLength
		TableLength = TableLength - 1
		
		i = i + 1
	end
	
	return TechArray
end

-- Same as ResearchTech except it returns the ExtraSlots value after executing a research
function ResearchTechExtraSlots(ai, ministerTag, ExtraSlots, TechArray)
	-- Performance check, am I realy suppose to do anything?
	if ExtraSlots <= 0 then
		return
	end

	local TableLength = table.getn(TechArray)
	local ResearchCount
	local TechSelected
	
	-- Just makes sure it can loop with no issues, the GetExtraSlots already account for it being larger
	if TableLength == 0 then
		return ExtraSlots
	elseif ExtraSlots >= TableLength then
		ResearchCount = TableLength
		ExtraSlots = ExtraSlots - TableLength
	else
		ResearchCount = ExtraSlots
		ExtraSlots = 0
	end
	
	local i = 0
	while i < ResearchCount do
		TechSelected = math.random(TableLength)

		local command = CStartResearchCommand( ministerTag, TechArray[TechSelected] )
		ai:Post( command )
		
		-- Remove the tech from the array
		table.remove(TechArray, TechSelected)
		-- Update the TabelLength
		TableLength = TableLength - 1
		
		i = i + 1
	end
	
	return ExtraSlots
end

-- Figures out if there are more slots available than techs
--   this would occur if there are more slots than techs the AI can research in the category area
function GetExtraSlots(ExtraSlots, ResearchCount, TechArray)
	if ResearchCount > 0 then
		TableLength = table.getn(TechArray)
		
		if ResearchCount > TableLength then
			ExtraSlots = ExtraSlots + (ResearchCount - TableLength)
		end
	end
	
	return ExtraSlots
end

